#ifndef M_LOG_H
#define M_LOG_H
/* Pxys - An IRC ProxyScan, m_log.h
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * $Id: m_log.h,v 1.4 2001/10/26 22:30:44 mbuna Exp $
 */

#include "typedefs.h"

#include <stdarg.h>
#include <time.h>

enum LogIDs {
   LOGID_PROXY,     /* Proxy logging (with info about the proxy, etc.) */
   LOGID_PXYS,      /* Pxys system logging */
   LOGID_GLINES,    /* Glines output logging */
   LOGID_PRIVMSG,   /* Incomming PRIVMSG */
   LOGID_OPERCMDS,  /* Opers commands logging */
   LOGID_LISTENER,  /* Listener's logging */
   LOGID_STATS      /* Statistics output */
};


/* Proto' */

void log_init();
void log_write(enum LogIDs id, const char *format, ...);
void log_vwrite(enum LogIDs id, const char *format, va_list vl);
void log_close();
int log_rotate_check(time_t now);
void log_rotate_sync();
int log_rotate_cut();
void log_rotate_hist(int resync);

void log_error(const char *format, ...);


/* Slacken logging */

#define LOGSLACK_ACCEPT_ECONNABORTED     1
#define LOGSLACK_GETPEERNAME_ECONNRESET  2
#define LOGSLACK_SCANTHREAD_NOTFOUND     3


struct log_slackitem {
   int id;          /* to identify */
   int dumped;      /* already written? */
   char buf[256];   /* string buffer */
};

typedef struct log_slackitem log_slackitem_t;

struct log_slackinfo {
   int size;                  /* number of items currently waiting to be logged */
   log_slackitem_t *items;    /* the items */
};

typedef struct log_slackinfo log_slackinfo_t;


/* Proto' */

void log_slacken_init();
void log_slacken_destroy();
void log_slacken(int slack_id, const char *format, ...);
void log_slacken_write();
void log_slacken_dump();


#endif  /* M_LOG_H */
