#ifndef M_STRUCTS_H
#define M_STRUCTS_H
/* Pxys - An IRC ProxyScan, m_structs.h
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * $Id: m_structs.h,v 1.9 2001/10/29 00:33:44 mbuna Exp $
 */

#include "config.h"


#include <sys/types.h>          /* time_t */
#include <netinet/in.h>         /* in_addr */

#include "numnicks.h"
#include "typedefs.h"

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#ifdef ENABLE_POLL_EMULATION
#include "poll.h"
#endif

#if !defined(ENABLE_POLL_EMULATION) && defined(HAVE_SYS_POLL_H)
#include <sys/poll.h>
#endif


#define SERVERTYPE_US				1
#define SERVERTYPE_EU				2
#define SERVERTYPE_NA				3

/* state defines of a scan thread */
#define STH_STATE_RUNNING			0x0001  /* Thread is scanning IPs. */
#define STH_STATE_WAITING			0x0002  /* This thread has nothing to scan. */

/* Scans which are "done" masks */
#define SCANTHREAD_DONE_WINGATE     0x0008
#define SCANTHREAD_DONE_SOCKS       0x0010
#define SCANTHREAD_DONE_HTTP3128    0x0020
#define SCANTHREAD_DONE_HTTP8080    0x0040
#define SCANTHREAD_DONE_HTTP80      0x0080

#define SCANTHREAD_RESCAN           0x0100
#define SCANTHREAD_TIMEWAIT         0x0200

#define SIT_INACTIVE                0x00
#define SIT_ACTIVE                  0x01
#define SIT_TIMEOUT                 0x02
#define SIT_ERROR                   0x04
#define SIT_INPROGRESS              0x08
#define SIT_RECEIVED                0x10
#define SIT_FOUND                   0x20
#define SIT_NOTFOUND                0x40
#define SIT_NOSOCKS                 0x80



#ifndef SCANS_PER_THREAD
#define SCANS_PER_THREAD            10
#endif

#ifndef NICKLEN
#define NICKLEN                     9 /* RFC? */
#endif
#define FLAG_NULL                   0x00000000
#define FLAG_IDENT                  0x00000001  /* User is idented (no ~) */
#define FLAG_OPER                   0x00000002  /* Global operator (+o) */

/* All (remote) IRC clients are stored in an array of struct Client */
struct Client
{
   unsigned int nserv;
   unsigned int nnick;
   char nick[NICKLEN+1];        /* auto nick[10] */ 
   unsigned int flags;          /* Flags: operator? ident'd?... */
   struct in_addr ip;           /* Real IP */
   time_t last;                 /* Last scanned */
};

/* The servers... */
struct Server
{
   void** cliH;                 /* Link to the clients table */
   
   char* name;                  /* Server name */
   unsigned int myx;            /* Server numeric */
   unsigned int upx;            /* Uplink numeric */
   unsigned int nn_mask;        /* Numeric nick mask */
   int noscan;                  /* No-Scan toggle */
   int shared;                  /* Shared toggle */
};


struct scanItem {
   unsigned int nserv;
   unsigned int nnick;
   struct in_addr addr;     
   unsigned short state;
   time_t timeout;
   struct sockaddr_in clientAddr;
   int tryread;
};


struct scanthread_s
{
   pthread_t threadID;            /* single thread ID */
   int counter;                   /* number of SIT_ACTIVE items */
   struct scanItem item[SCANS_PER_THREAD];
   struct pollfd poll_fds[SCANS_PER_THREAD];
   unsigned int state;            /* Customized scanthread state. */
   pthread_cond_t sem;            /* semaphore used by the listener to tell the appropriate
                                   * thread that the proxy has replied (vulnerable CONNECT..)
                                   */
   struct scanthread_s *next;     /* the next thread info */
};


typedef struct scanthread_s scanthread_s;


struct scanstats_s {
   int is_sync;                   /* Is the pxys "sync", ie. no need to work at 100% */
   int running_threads;           /* Number of running threads */
   int waiting_threads;           /* Number of waiting threads */
   
};

typedef struct scanstats_s scanstats_s;


/* --------------------------- Conf structs --------------------------- */

/* Sizes */
#define MAXSERVERNAME_LENGTH	64
#define MAXSERVERINFO_LENGTH	64
#define MAXPASSWORD_LENGTH		32
#define MAXNICKNAME_LENGTH		10	/* 9+1 */
#define MAXUSERNAME_LENGTH		11	/* 10+1 */
#define MAXREALNAME_LENGTH		64
#define MAXFILENAME_LENGTH		32

/* me {} */
struct Me_s {
	char name[MAXSERVERNAME_LENGTH];
	long numeric;
	long capacity;
	char serverinfo[MAXSERVERINFO_LENGTH];
	char hostip[16];
	Boolean vhost;     /* vhost for the scanner? */
	char bindip[16];
	unsigned long bindaddr;
	char listener[16];
	unsigned long listeneraddr;
	char id[10];
};

typedef struct Me_s 	Me_s;


/* uplink {} */
struct Uplink_s {
	char host[MAXSERVERNAME_LENGTH];
	Boolean vhost;     /* vhost for the hub connection? */
	char bindip[16];
	unsigned long bindaddr;
	unsigned short port;
	char password[MAXPASSWORD_LENGTH];
	int priority;
	struct Uplink_s* next;		/* Several uplink servers supported. */
};

typedef struct Uplink_s 	Uplink_s;


/* console {} */
struct Console_s {
	char * channel;
	char mode[16];
};								/* Several console channels not supported. */

typedef struct Console_s 	Console_s;


/* client {} */
struct MyClient_s {
	char nickname[MAXNICKNAME_LENGTH];
	char username[MAXUSERNAME_LENGTH];
	char * hostname;
	char * realname;
	char mode[16];
	struct in_addr ip;
	Boolean join;
	char nick[MAXNICKNAME_LENGTH];
};	

typedef struct MyClient_s 	MyClient_s;


/* perform {} */
struct Perform_s {
	long scanthreads;
	long recheck;
	long recheckshared;
	long timeout_connect;
	long timeout_send;
	long timeout_listen;
	long cachesize;
};

typedef struct Perform_s 	Perform_s;


/* share {} */
struct Share_s {
   char name[MAXSERVERNAME_LENGTH];
   char successor[MAXSERVERNAME_LENGTH];
   struct Serverlist* servlist;
   struct Share_s *next;            /* More than 2 proxyscan sharing supported. */
};

typedef struct Share_s 	Share_s;


/* noscan {} */
struct NoScan_s {
	struct IPrangelist *		ips;
	struct Serverlist *			residences;
};

typedef struct NoScan_s 	NoScan_s;


struct Log_s {
    int update;
    char dir[MAXFILENAME_LENGTH];
    char history_dir[MAXFILENAME_LENGTH];
    
	char current[MAXFILENAME_LENGTH];
	char daily[MAXFILENAME_LENGTH];
	
	char pxys[MAXFILENAME_LENGTH];
	char glines[MAXFILENAME_LENGTH];
	char privmsg[MAXFILENAME_LENGTH];
	char opercmds[MAXFILENAME_LENGTH];
	char listener[MAXFILENAME_LENGTH];
	char stats[MAXFILENAME_LENGTH];
};

typedef struct Log_s 	Log_s;



/* Used for socks4 check: Some target server to test the insecurity */
struct Target_s {
	char name[MAXSERVERNAME_LENGTH];
	struct in_addr addr;
	struct Target_s * next;
};

typedef struct Target_s 	Target_s;


struct Gline_s {
	int direct;
	time_t delay;
	time_t cached;
	char reason_wingate[160];
	char reason_socks[160];
	char reason_http3128[160];
	char reason_http8080[160];
	char reason_http80[160];
	struct Relay_s *relay;
};

typedef struct Gline_s 	Gline_s;


/* Sub-structs */

struct Serverlist {
	char name[MAXSERVERNAME_LENGTH];
	struct Serverlist* next;
};

typedef struct Serverlist 	Serverlist;


struct IPrangelist {
	struct in_addr 		start;
	struct in_addr 		stop;
	struct IPrangelist* next;
};

typedef struct IPrangelist 	IPrangelist;


struct Relay_s {
	char nick[NICKLEN+1];
	char serv[MAXSERVERNAME_LENGTH];
        char auth[64];
        Boolean isauth;
	struct Relay_s * next;
};

typedef struct Relay_s 	Relay_s;


#endif  /* M_STRUCTS_H */
