/* Pxys - An IRC ProxyScan, m_clientq.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * A "nicks" queue of recent connected users on the network. This is used
 * by m_array to optimize the fetchitem methods (often called by the scan
 * threads), when the proxyscan is sync (ie. when it scans only new users).
 *
 * $Id: m_clientq.c,v 1.4 2001/10/31 20:49:19 mbuna Exp $
 */

#include "m_clientq.h"

#ifdef CLIENTQ_TEST
#define xcalloc calloc
#define xfree free
#define CLIENTQ_SIZE 100
#define CLIENTQ_INJECTSIZE 10000000
#else
#include "config.h"
#include "xmalloc.h"
#include "m_structs.h"
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef CLIENTQ_TEST
extern scanstats_s *scanstats;
#endif

static int aepos, aesize;

/* pointers */
static listelement *aelem;
static listelement *listptr;

/* optimization: pointer to the last allocated element of the list:
 * without:   28.230u 0.440s 0:48.17 59.5%    0+0k 0+3io 0pf+0w
 * with listq: 4.490u 0.050s 0:06.56 69.2%     0+0k 0+3io 0pf+0w
 * CLIENTQ_SIZE=100 CLIENTQ_INJECTSIZE=10000000
 */
static listelement *listq;

/* Pre-allocate */
void
clientq_init()
{
   listptr = NULL; listq = NULL;
   aepos = 0; aesize = 0;
#if CLIENTQ_SIZE > 0
   /* printf("clientq_init: %d elements\n", CLIENTQ_SIZE); */
   aelem = (listelement *)xcalloc(CLIENTQ_SIZE, sizeof(listelement));
#endif
}

void
clientq_destroy()
{
#if CLIENTQ_SIZE > 0
   xfree(aelem);
#endif
}

#if CLIENTQ_SIZE > 0

/* nick != NULL */
void
clientq_queue(const char *nick)
{
   listelement * lp = listptr;
   
   if (aesize >= CLIENTQ_SIZE) {
      clientq_deque(NULL);
#ifndef CLIENTQ_TEST
      scanstats->is_sync = 0;
#endif
   }
   
   if (lp != NULL) {
      lp = listq;
      lp->link = &aelem[aepos];
      lp = lp->link;
   } else {
      listptr = &aelem[aepos];
      lp = listptr;
   }
   
   lp->link = NULL;
   strncpy(lp->nick, nick, NICKLEN);
   lp->nick[NICKLEN] = '\0';
   
   listq = lp;
   
   aepos = (aepos < CLIENTQ_SIZE - 1) ? aepos + 1 : 0 ;
   aesize++;
}

#else
void clientq_queue(const char *nick) {}
#endif


int
clientq_deque(char **out_nick)
{
   if (listptr) {
      if (out_nick) /* Allow NULL, no output */
         *out_nick = listptr->nick;
      listptr = listptr->link;
      aesize--;
      return 1;
   }
   return 0; /* nothing in queue */
}


void
clientq_stats(int* osize, int* oaesize)
{
   *osize = CLIENTQ_SIZE;
   *oaesize = aesize;
   
}

#ifdef CLIENTQ_TEST
int
main(void)
{
   char *nick;
   int i;
   
   clientq_init();
   
   for (i = CLIENTQ_INJECTSIZE; i--; )
      clientq_queue("nick1");
   
   while(clientq_deque(&nick))
      ;
   
   clientq_destroy();
   return 0;
}

#endif


/* m_clientq.c */
