/* Pxys - An IRC ProxyScan, m_scanner.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 *
 *
 * Scanner node.
 *
 * $Id: m_scanner.c,v 1.13 2001/10/30 10:49:39 mbuna Exp $
 */

#define NOJOB_NOTICEDELAY 3600

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "conf.h"
#include "debug.h"

#include "typedefs.h"

#include "m_array.h"
#include "m_log.h"
#include "m_noscan.h"
#include "m_scanner.h"
#include "m_share.h"
#include "m_stats.h"
#include "m_thread.h"

#include "ms_globals.h"
#include "ms_msgout.h"       /* to inform the console channel */
#include "ms_netutils.h"

#include "mp_proxy.h"
#include "mp_socks.h"
#include "mp_wingate.h"


extern pthread_mutex_t mutex_init;
extern pthread_mutex_t mutex_scanext;

long totalscanned = 0;
long totalfound = 0;

extern scanstats_s *scanstats;
extern time_t startscan_time;
time_t lastnojob = 0;
Boolean syncsent = false;


void
start_scan()
{
   scanthread_s *th;
   char locbuf[0x80];
   Boolean rescan;
   struct timespec timed;
   pthread_mutex_t mutex_dummy;
   time_t now;
   
   pthread_mutex_init(&mutex_dummy, NULL);
   pthread_mutex_lock(&mutex_dummy) ;
   
   Debug((DBLBASIC, "start_scan> %lu", (unsigned long) pthread_self()));
   
   pthread_mutex_lock(&mutex_init);
   th = get_scanthread(pthread_self());
   pthread_mutex_unlock(&mutex_init);
   
   if(NULL == th) {
      fprintf(stderr, "start_scan> th == NULL ? (%lu)\n", (unsigned long)pthread_self());
      exit(EXIT_FAILURE);
   }
   
   pthread_cond_wait(&th->sem, &mutex_dummy);
   pthread_cond_destroy(&th->sem);
   
   Debug((DBLBASIC, "start_scan> (%ld) Endless loop...", (unsigned long) pthread_self()));
   
   /* Endless loop... */
   for(;;) {
      
      pthread_mutex_lock(&mutex_scanext);
      
      array_fetchitem(th);
      
      pthread_mutex_unlock(&mutex_scanext);  /* Allow others threads to get a next IP */
      
      
      /*
      for(i=0; i<SCANS_PER_THREAD; i++) {
         if(th->item[ i ].state == SIT_ACTIVE) {
            Debug((DBLBASIC, "client in packet: %s", inet_ntoa(th->item[ i ].addr)));
         }
      }
      */
      
      if(th->counter == 0) {
         
         if(scanstats->waiting_threads >= 1) {
            pthread_mutex_lock(&mutex_init);  /* test&set */
            if(!syncsent) {
               syncsent = true;
               pthread_mutex_unlock(&mutex_init);
               time(&now);
               human_time(now - startscan_time, locbuf);
               sendto_console("[%s] Network sync done. (%s)", Me->id, locbuf) ;
            } else
               pthread_mutex_unlock(&mutex_init);
         }
         
         if(scanstats->running_threads == 0) {
            pthread_mutex_lock(&mutex_init);  /* test&set */
            time(&now);
            if((now - lastnojob) > NOJOB_NOTICEDELAY) {
               lastnojob = now;
               pthread_mutex_unlock(&mutex_init);
               human_time(now - startscan_time, locbuf);
               sendto_console("[%s] No job remaining, waiting... (up: %s)", Me->id, locbuf) ;
            } else
               pthread_mutex_unlock(&mutex_init);
         }
         
         /* Wait 3 seconds */
         
         time(&timed.tv_sec);
         timed.tv_sec += 3;
         timed.tv_nsec = 0;
         pthread_cond_init(&th->sem, NULL);
         pthread_cond_timedwait(&th->sem, &mutex_dummy, &timed);
         pthread_cond_destroy(&th->sem);
         
         /* sleep(3);  <- bad with pthreads :) */
         
      } else {
         
         Debug((DBLMAX, "start_scan: OK scan..."));
         
         
         do {
            
            if(th->state & SCANTHREAD_RESCAN)
               th->state |= ~SCANTHREAD_RESCAN;
            
#ifdef SCAN_WINGATE
            // WinGate scan
            if(!(th->state & SCANTHREAD_DONE_WINGATE))
               scan_wingate(th);
            th->state |= SCANTHREAD_DONE_WINGATE;
#endif
#ifdef SCAN_SOCKS
            // Socks4+5 scan
            if(!(th->state & SCANTHREAD_DONE_SOCKS))
               scan_socks(th);
            th->state |= SCANTHREAD_DONE_SOCKS;
#endif
#ifdef SCAN_HTTP3128
            // Http proxy scan, port 3128
            if(!(th->state & SCANTHREAD_DONE_HTTP3128))
               scan_proxy(th, 3128);
            th->state |= SCANTHREAD_DONE_HTTP3128;
#endif
#ifdef SCAN_HTTP8080
            // Http proxy scan, port 8080
            if(!(th->state & SCANTHREAD_DONE_HTTP8080))
               scan_proxy(th, 8080);
            th->state |= SCANTHREAD_DONE_HTTP8080;
#endif
#ifdef SCAN_HTTP80
            // Http proxy scan, port 80
            if(!(th->state & SCANTHREAD_DONE_HTTP80))
               scan_proxy(th, 80);
            th->state |= SCANTHREAD_DONE_HTTP8080;
#endif
            rescan = th->state & SCANTHREAD_RESCAN;
            
         } while(rescan);
         
         totalscanned += th->counter;
      }
   }
}




/* m_scanner.c */
