/* Pxys - An IRC ProxyScan, m_thread.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * Threads init and stuffs.
 *
 * $Id: m_thread.c,v 1.11 2001/10/26 22:30:44 mbuna Exp $
 */


#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif
#include <signal.h>
#include <errno.h>
#include <string.h>

#include "typedefs.h"
#include "conf.h"
#include "debug.h"
#include "xmalloc.h"

#include "m_thread.h"
#include "m_scanner.h"
#include "m_array.h"
#include "m_log.h"
#include "m_shutdown.h"

#include "ms_globals.h"
#include "ms_msgout.h"

#include "mp_listener.h"


pthread_mutex_t mutex_init;
pthread_mutex_t mutex_server;
extern pthread_mutex_t mutex_scanext;
extern pthread_mutex_t mutex_link;

scanthread_s *scanthread = NULL;
scanstats_s *scanstats = NULL;

/* Allocate the threads */
void
scanner_init()
{

   int i;
   pthread_t newthread;
   scanthread_s *scanth;
   scanthread_s *scanth_last = NULL;
   
   /* Stats */
   scanstats = (scanstats_s *)xmalloc(sizeof(scanstats_s));
   scanstats->is_sync = 0;
   scanstats->running_threads = 0;
   scanstats->waiting_threads = MyPerform->scanthreads;
   
   
   pthread_mutex_init(&mutex_init, NULL);
   pthread_mutex_lock(&mutex_init);
   
   pthread_mutex_init(&mutex_server, NULL);
   scanthread = NULL;
   
   printf("Initializing listener.\n");
   listener_init();
   
   /* Create the listener (server) in a new thread */
   if (pthread_create(&newthread, NULL, RunServer, NULL)) {
      fprintf(stderr, "Server creation failed. Aborting.\n");
      exit(EXIT_FAILURE);
   }
   
   /* Wait for listener thread to detach */
   pthread_join(newthread, NULL);
   
   Debug((DBLMAX, "init_scanner> Listener initialized. Initializing scanthreads..."));
   
   printf("Initializing scanning threads.\n");
   
   for(i = 0; i < MyPerform->scanthreads; i++) {
      if(!pthread_create(&newthread, NULL, RunScanner, NULL)) {
         
         Debug((DBLMAX, "scanner_init> th: %lu", (unsigned long) newthread));
         
         scanth = (scanthread_s *) xmalloc(sizeof(scanthread_s));
         memset(scanth, 0, sizeof(scanthread_s));
         
         scanth->threadID = newthread;
         scanth->state = STH_STATE_WAITING;
         pthread_cond_init(&scanth->sem, NULL);
         scanth->next = NULL;
         
         if (NULL == scanth_last)
            scanthread = scanth;
         else
            scanth_last->next = scanth;

         scanth_last = scanth;
         
      } else {
         fprintf
            (stderr, "\n\nERROR! Not enough system resources to create a new thread "
            "or too much threads; please use a lower value for scanthreads.\n");
         log_error("m_thread:scanner_init: new thread failed (%d/%d)\n",
              i, MyPerform->scanthreads);
         exit(1);
      }
   }
   
   pthread_mutex_unlock(&mutex_init);
}

void
scanner_startall()
{
   scanthread_s *th = scanthread;
	
   Debug((DBLMAX, "scanner_startall"));

#ifdef HAVE_PTHREAD_COND_BROADCAST
   pthread_cond_broadcast(&th->sem);
#else
   for(; th; th=th->next)
      pthread_cond_signal(&th->sem);
#endif
}

/* scanner_killall is mainly for linuxthreads */
void
scanner_killall()
{
#ifdef HAVE_PTHREAD_KILL
   scanthread_s *th = scanthread;
   
   for(; th; th=th->next)
      pthread_kill(th->threadID, SIGKILL);
#endif
}


/* Execute a scan thread. */
void*
RunScanner()
{
   start_scan();
   return NULL;
}

/* Launch the server (which will listen for the "CONNECT" replies) */
void*
RunServer()
{
   listener_run();
   return NULL;
}


scanthread_s *
get_scanthread(pthread_t thread)
{
   scanthread_s *ptr;
   
   for(ptr = scanthread; ptr; ptr = ptr->next) {
      if(ptr->threadID == thread)
         return ptr;
   }

   return NULL;                 /* Not found. Argh. */
}


scanthread_s *
get_scanthread_from_ip(struct in_addr * ip, int *outport)
{
   int i;
   scanthread_s *th = NULL;
   
   pthread_mutex_lock(&mutex_scanext);
   /* for each thread */
   for(th=scanthread; th; th=th->next) {
      if(th->state & STH_STATE_RUNNING) {
         /* for each item */
         for(i=0; i<th->counter; i++) {
            if((th->item[i].state & SIT_INPROGRESS) &&
               (memcmp(&th->item[i].addr, ip, sizeof(struct in_addr)) == 0)) {
               if(!(th->item[i].state & SIT_FOUND))
                  th->item[i].state |= SIT_FOUND;
               /* heh, we need only 1 reply! */
               if(th->item[i].state & SIT_RECEIVED) {
                  th = NULL;
               } else {
                  th->item[i].state |= SIT_RECEIVED;
                  if (th->state & SCANTHREAD_DONE_HTTP8080)
                     *outport = 80;
                  else if (th->state & SCANTHREAD_DONE_HTTP3128)
                     *outport = 8080;
                  else 
                     *outport = 3128;
               }
               goto exit_loop;
            }
         }
      }
   }
exit_loop:
   pthread_mutex_unlock(&mutex_scanext);
   return th;
}

/* Called by mp_listener when changing the listening port on the fly. */
void
set_globalrescan()
{
   scanthread_s *ptr;

   for(ptr = scanthread; ptr; ptr = ptr->next) {
      if((ptr->state & STH_STATE_RUNNING) && !(ptr->state & SCANTHREAD_RESCAN)) {
         ptr->state &= ~SCANTHREAD_DONE_HTTP3128 & ~SCANTHREAD_DONE_HTTP8080;
         ptr->state |= SCANTHREAD_RESCAN;
      }
   }
}



/* m_thread.c */
