/* Pxys, An IRC ProxyScan, mp_timers.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 *
 * $Id: m_timers.c,v 1.8 2001/11/01 01:34:08 mbuna Exp $
 */



#include "config.h"

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "typedefs.h"
#include "debug.h"
#include "conf.h"
#include "m_cache.h"
#include "m_log.h"    /* log_check */
#include "m_noscan.h"
#include "m_proxycache.h"
#include "m_share.h"
#include "m_stats.h"
#include "m_thread.h"
#include "m_timers.h"
#include "ms_auth.h"
#include "ms_globals.h"
#include "ms_msgout.h"
#include "ms_network.h"

time_t timer;
time_t timer_ping;
time_t timer_stats;
time_t timer_consolestats;
time_t timer_nickchange;
time_t timer_sync_proxycache;
time_t timer_cache_dump;
time_t timer_network_recheck; /* Used to regularly rescan the network even if sync'd */

int gonnaRehash;

extern pthread_mutex_t mutex_scanext;
extern pthread_mutex_t mutex_server;
extern scanstats_s *scanstats;

void
m_timers_init()
{
   time_t now;
   
   time(&now);
   
   timer_ping            = now + PING_DELAY;
   timer_stats           = now + 2;
   timer_consolestats    = now + CONSOLESTATS_DELAY;
   timer_nickchange      = now + AUTO_NICKCHANGE_TIME;
   timer_sync_proxycache = now + RESYNC_PROXYCACHE_DELAY;
/*   timer_spread_glines   = 0; */
   timer_cache_dump      = now + AUTO_CACHE_DUMP_DELAY;
   timer_network_recheck  = 0;
   
   gonnaRehash           = 0;
}

void
m_timers_check()
{
   int n;
   time_t now;
   
   time(&now);
   
   if (now == timer) /* return if not at least 1 sec has elapsed */
      return;
   
   if (gonnaRehash > 0)
      m_rehash();
   
   log_slacken_write(); /* Write "slacken" logging from mp_listener */
   
   					/* Eventually, sync log files. */
   n = log_rotate_check(now);
   if (n >= 0)
      sendto_console("[%s] Rotating log files (%d proxies)", Me->id, n);
   
   if (scanstats->is_sync) {
      if (timer_network_recheck == 0)
         timer_network_recheck = now + NETWORK_RECHECK_DELAY;
      else if (now >= timer_network_recheck) {
         timer_network_recheck = 0;
         scanstats->is_sync = 0; /* volontary desync it */
      }
   }
   
   					/* Update stats records every 5 min */
   if(now >= timer_stats) {
      timer_stats = now + 300;
      stats_periodical_record(now);
   }
   
   					/* Console stats */
   if(now >= timer_consolestats) {
      timer_consolestats = now + CONSOLESTATS_DELAY;
      consolestats();
   }
   
   if (now >= timer_cache_dump) {
      timer_cache_dump = now + AUTO_CACHE_DUMP_DELAY;
      cache_dump();
      return;
   }
   					/* Auto nick change */
   if(now >= timer_nickchange) {
      timer_nickchange = now + AUTO_NICKCHANGE_TIME;
      timer_ping = now + PING_DELAY;
      do_nickchange();
   }
   
   					/* Additional pings */
   if(now >= timer_ping) {
      timer_ping = now + PING_DELAY;
      sendto_uplink("%s %s :%s", MyYY, TOK_PING, Me->name);
   }
   
   if(now >= timer_sync_proxycache) {
      timer_sync_proxycache = now + RESYNC_PROXYCACHE_DELAY;
      proxycache_syncfile();
   }
   
/* Glines refreshing (spreading across the network)
 *
 * 3 modes for gonnaSpreadGlines
 *    0 : no need to refresh glines
 *    1 : currently refreshing glines
 *    2 : ie. after a net junction, we are going to spread the g's
 *
 * When gonnaSpreadGlines == 1, glines_spread is called (about) every second
 * in order to avoid the flood when sending the glines to our uplink.
 * glines_spread() refreshs only a few glines each time.
 */
/*
   if (!gonnaSpreadGlines || !MyGline->direct)
      return;
   
   if (gonnaSpreadGlines == 1) {
      Debug((DBLMAX, "mp_chktimers: glines_spread() [1]"));
      gonnaSpreadGlines = glines_spread();
      
   } else if ((gonnaSpreadGlines == 2)
      && ((now >= timer_spread_glines) >= SPREAD_GLINES_DELAY)) {
      timer_spread_glines = now + SPREAD_GLINES_DELAY;
      Debug((DBLMAX, "mp_chktimers: glines_spread() [2]"));
      gonnaSpreadGlines = glines_spread();
   }
   */
}


void
m_rehash()
{
    /* Limit threads activity */
   pthread_mutex_lock(&mutex_scanext);
   pthread_mutex_lock(&mutex_server);
   
   if (conf_reload())
      sendto_console("Reloading configuration file: successful.") ;
   else
      sendto_console("Reloading configuration file: failed.") ;
   
   conf_resolv();
   
   refresh_shared_servers();
   refresh_noscan_servers();
   refresh_relay_services();
   
   sendto_console("Refreshing shared/noscan servers: done.") ;
   
   gonnaRehash = 0;
   
   pthread_mutex_unlock(&mutex_server);
   pthread_mutex_unlock(&mutex_scanext);
}


/* mp_alarm.c */
