/* Pxys - An IRC ProxyScan, mp_proxy.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * 
 * Module checking proxy/webserver tunelling vulnerability.
 *
 * $Id: mp_proxy.c,v 1.5 2001/10/30 11:39:48 mbuna Exp $
 */

#define HTTP_CONNECT_REQUEST "CONNECT %s:%d/ HTTP/1.1\nHost:%s:%d\n\n"

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "typedefs.h"
#include "conf.h"
#include "mp_proxy.h"
#include "mp_listener.h"
#include "mp_utils.h"
#include "ms_globals.h"
#include "ms_msgout.h"
#include "m_log.h"
#include "m_thread.h"
#include "debug.h"
#include "conf.h"
#include "mp_poll.h"   /* poll macros */

#include <errno.h>

extern int SPort;

extern pthread_mutex_t mutex_scanext;

void
scan_proxy(scanthread_s * sth, int port)
{
   int err, nfds, i;
   struct pollfd *pfd;
   char request[96];
   struct timespec timewait;
   pthread_mutex_t mutex_dummy;
   time_t now;
   unsigned int length;
   Boolean oneinprogress = false;
   Boolean stilloneinprogress = false;
   Boolean timed_out;
   
   /* - Mutex initialization */
   pthread_mutex_init(&mutex_dummy, NULL);
   pthread_mutex_lock(&mutex_dummy) ;
   
   /* - Building http proxy CONNECT request */
   if (Me->listeneraddr == htonl(INADDR_ANY))
      snprintf(request, sizeof(request), HTTP_CONNECT_REQUEST,
              Me->hostip, SPort, Me->hostip, SPort);
   else
      snprintf(request, sizeof(request), HTTP_CONNECT_REQUEST,
              Me->listener, SPort, Me->hostip, SPort);
           
   scanItems_init(sth, POLLWRITEFLAGS, (unsigned short)port);
   
   for(;;) {
      
      nfds = poll(sth->poll_fds, sth->counter, POLL_TIMEOUT);
      
      if(nfds <= -1) {
         if(errno == EINTR)
            continue;
         log_error("scan_proxy: poll error %d\n", errno);
         break;
      }
      
      /* Timeout checking */
      time(&now);
      timed_out = true;
      for(i=0; i < sth->counter; ++i)
      {
         if(sth->item[i].state & (SIT_INACTIVE|SIT_ERROR|SIT_FOUND|SIT_INPROGRESS))
            continue;
         
         if(!(sth->item[i].state & SIT_TIMEOUT) && (now >= sth->item[i].timeout))
         {
            sth->item[i].state |= SIT_TIMEOUT;
         }
         
         timed_out = timed_out && ( sth->item[i].state & SIT_TIMEOUT );
      }
      
      if(timed_out)
      {
         /* Every fds are timed out, exiting. */
         Debug((DBLMAX, "scan_proxy: all timed out"));
         break;
      }
      
      if(nfds == 0)
        continue;
       
      pfd = sth->poll_fds;
      
      for (i=0 ; (i < sth->counter); ++i, ++pfd)
      {
         if(pfd->revents & POLLERRORS) {
            sth->item[i].state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            continue;
         }
         
         if(sth->item[i].state & (SIT_INACTIVE|SIT_ERROR|SIT_FOUND|SIT_INPROGRESS))
            continue;
         
         if((err = get_sockerr(pfd->fd))) {
            sth->item[i].state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            continue;
         }
         
         if(pfd->revents & POLLWRITEFLAGS)
         {
            /* Write should not block or failed... let's go. */
            Debug((DBLMAX, "scan_proxy: send_nonb..."));
            
            /* Eventually, set a new timeout delay. */
            time(&sth->item[i].timeout);
            sth->item[i].timeout += MyPerform->timeout_send;
            if(send_nonb(pfd->fd, request, strlen(request) + 1, &length) < 0)
            {
               Debug((DBLMAX, "scan_proxy: send_nonb failed"));
               sth->item[i].state |= SIT_ERROR;
               DISABLE_PFD(pfd);
               continue;
            }
            sth->item[i].state |= SIT_INPROGRESS;
            oneinprogress = true; /* to avoid items check (faster if nothing sent) */
            DISABLE_PFD(pfd);
         }
      }
   }
   
   if(oneinprogress) {
      
      stilloneinprogress = false;
      for (i=0 ; (i < sth->counter); ++i) {
         if((sth->item[i].state & SIT_INPROGRESS) && !(sth->item[i].state & SIT_RECEIVED))
         {
            stilloneinprogress = true;
            break;
         }
      }
      
      if(stilloneinprogress)
      {
         pthread_cond_init(&sth->sem, NULL);  /* Create semaphore */
         time(&timewait.tv_sec); /* Reentrant */
         timewait.tv_sec += MyPerform->timeout_listen;
         timewait.tv_nsec = 0;
         
         sth->state |= SCANTHREAD_TIMEWAIT;
         
         do {
            err = pthread_cond_timedwait(&sth->sem, &mutex_dummy, &timewait);
            if (err == ETIMEDOUT) {
               Debug((DBLMAX, "scan_proxy: timeout"));
               break;
            }
            
            Debug((DBLMAX, "scan_proxy: Reply received!!!"));
            
            stilloneinprogress = false;
            for (i=0 ; (i < sth->counter); ++i) {
               if((sth->item[i].state & SIT_INPROGRESS) && !(sth->item[i].state & SIT_RECEIVED))
               {
                  stilloneinprogress = true;
                  break;
               }
            }
         } while(stilloneinprogress);
      
         sth->state &= ~SCANTHREAD_TIMEWAIT;
         pthread_cond_destroy(&sth->sem);
      }
   }
   
   /* mass close :P~ */
   scanItems_closeall(sth);
   Debug((DBLMAX, "scan_proxy: scanItems_closeall"));
}


/* m_proxy.c */
