/* Pxys - An IRC ProxyScan, mp_socks.c
 * 
 * Copyright (C) 2001 Stephane Thiell
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * E-mail of the author: mbuna@undernet.org
 *
 *
 * Unsecure socks servers version 4 and 5 scanner.
 *
 * $Id: mp_socks.c,v 1.13 2001/11/04 01:53:37 mbuna Exp $
 */


#define SOCKS_PORT      1080
#define SOCKS4_USERID   "dummy"

/* Removed TOAST_REJECTED_SOCKS4, now, I'm glining every replyed socks replied
 * as socks v4 with an status code != rejected. It should avoid tadd's gline :P~
 */


#include "config.h"

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include <errno.h>

#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "conf.h"
#include "debug.h"

#include "typedefs.h"

#include "mp_socks.h"
#include "mp_utils.h"
#include "mp_poll.h"

#include "m_log.h"
#include "m_thread.h"

#include "ms_globals.h"
#include "ms_msgout.h"

#include "socks4.h"             /* Socks4 protocol constants */
#include "socks5.h"             /* Socks5 protocol constants */

extern pthread_mutex_t mutex_scanext;


/* Why we cannot check socks version 4 then version5 in one connection:

The SOCKS server closes its connection immediately after notifying
the client of a failed or rejected request. For a successful request,
the SOCKS server gets ready to relay traffic on both directions. This
enables the client to do I/O on its connection as if it were directly
connected to the application server.

http://www.socks.nec.com/protocol/socks4.protocol

*/

void
scan_socks(scanthread_s *sth)
{
   int nfds, i, err;
   struct pollfd *pfd;
   unsigned int length;
   time_t now;
   int timed_out;
   
   /* Socks4 data init */
   struct socks4 socks4out = { S4_VN, S4_CMD_CONNECT, { 0x1A, 0x0B }, { 0, 0, 0, 0 }, 0 };
   struct socks4_response socks4in = { 0, 0, { 0, 0 }, { 0, 0, 0, 0 } };
   
   /* Socks5 data init */
   struct socks5 socks5out = { S5_VN, S5_CMD_CONNECT };
   struct socks5_response socks5in = { 0, 0 };
   
   /* To set the DST IP of the socks4 CONNECT request, we choose in the MyTarget list.
    * ie. if for Undernet we put an Undernet servers list, we are 100% sure that if
    * the request is granted by the SOCKS 4, it's misconfigured.
    * Without an Undernet server IP for the Undernet network, it would only be speculation.
    *
    * One day after: OK, this code (get_socks4_dest_ip) is in fact unuseful. SOCKS4 will
    * often reject the connection request, see the note for TOAST_REJECTED_SOCKS4 (at
    * the top of this page). So, even if rejected (but only if accepted OR rejected,
    * others reply are not considered as insecure SOCKS 4 proxy), we toast 'em.  --mbuna
    *
    * We let get_socks4_dest_ip for the fun (and because accepted request may hit a
    * specified server... with more servers, it's better).
    */
   
   get_socks4_dest_ip(&socks4out);
   
   /************************* SOCKS 4 CHECK ********************************/
   
   
   scanItems_init(sth, POLLWRITEFLAGS, SOCKS_PORT);
   
   Debug((DBLMAX, "scan_socks (SOCKS 4) sscanItems_init ok"));
   
   for( ; ; ) {
      
      nfds = poll(sth->poll_fds, sth->counter, POLL_TIMEOUT);
      
      if(nfds <= -1) {
         if(errno == EINTR)
            continue;
         log_error("scan_socks4: poll error (%d)\n", errno);
         break;
      }
      
      /* Timeout checking */
      time(&now);
      timed_out = true;
      for(i = 0; i < sth->counter; ++i)
      {
         if(sth->item[i].state & (SIT_INACTIVE|SIT_ERROR|SIT_FOUND))
            continue;
         
         if(!(sth->item[i].state & SIT_TIMEOUT) && (now >= sth->item[i].timeout))
         {
            sth->item[i].state |= SIT_TIMEOUT;
         }
         
         timed_out = timed_out && ( sth->item[i].state & SIT_TIMEOUT );
      }
      
      if(timed_out)
      {
         /* Every fds are timed out, exiting. */
         //Debug((DBLMAX, "scan_socks4: all timed out"));
         break;
      }
      
      if(nfds == 0)
         continue;
      
      pfd = sth->poll_fds;
      
      for (i = 0; i < sth->counter; ++i, ++pfd)
      {
         if(pfd->revents & POLLERRORS) {
            sth->item[i].state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            continue;
         }
         
         if(sth->item[i].state & (SIT_INACTIVE|SIT_ERROR|SIT_FOUND))
            continue;
         
         if((err = get_sockerr(pfd->fd))) {
            Debug((DBLMAX, "scan_socks4: %s get_sockerr", inet_ntoa(sth->item[i].addr)));
            sth->item[i].state |= SIT_ERROR | SIT_NOSOCKS;
            DISABLE_PFD(pfd);
            continue;
         }
         
         if((pfd->events & POLLWRITEFLAGS) && (pfd->revents & POLLWRITEFLAGS))
         {
            Debug((DBLMAX, "scan_socks4: %s write ready (%d)", inet_ntoa(sth->item[i].addr), i));
            time(&sth->item[i].timeout); /* Reentrant */
            sth->item[i].timeout += MyPerform->timeout_send;
            
            Debug((DBLMAX, "->%s %X %X %X %X %X %X %X %X %X", inet_ntoa(sth->item[i].addr),
               socks4out.vn, socks4out.cd, socks4out.dstport[0], socks4out.dstport[1],
               socks4out.dstip[0], socks4out.dstip[1], socks4out.dstip[2], socks4out.dstip[3],
               socks4out.nullfield));
            
            if(send_nonb(pfd->fd, (char *) &socks4out, sizeof(struct socks4), &length) < 0)
            {
               Debug((DBLMAX, "scan_socks4: %s send_nonb failed", inet_ntoa(sth->item[i].addr)));
               sth->item[i].state |= SIT_ERROR | SIT_NOSOCKS;
               DISABLE_PFD(pfd);
               continue;
            }
            else
            {
               pfd->events = POLLREADFLAGS;   /* Now we want to read the reply. */
               pfd->revents = 0;
               continue;
            }
         }
         
         if((pfd->events & POLLREADFLAGS) && (pfd->revents & POLLREADFLAGS))
         {
            
            memset((void *)&socks4in, 0, sizeof(struct socks4_response));
            if(recv_nonb(pfd->fd, (char *) &socks4in, sizeof(struct socks4_response), &length) < 0)
            {
               Debug((DBLMAX, "scan_socks4: %s recv_nonb failed", inet_ntoa(sth->item[i].addr)));
               sth->item[i].state |= SIT_ERROR;
               DISABLE_PFD(pfd);
               continue;
            }
            
            if(length > 1) {
               Debug((DBLMAX,
               "scan_socks4: %s recv_nonb %d (VN=%d CD=%d dstport = %d %d dstip = %d.%d.%d.%d)",
                  inet_ntoa(sth->item[i].addr), length,
                  (int)socks4in.vn, (int)socks4in.cd,
                  (int)socks4in.dstport[0], (int)socks4in.dstport[1],
                  (int)socks4in.dstip[0], (int)socks4in.dstip[1],
                  (int)socks4in.dstip[2], (int)socks4in.dstip[3]));
               
               /* Valid socks4 reply with granted access */
               if (socks4in.cd == S4_REQ_GRANTED) {
                  sth->item[i].state |= SIT_FOUND;
                  /* Toast it. */
                  proxy_toast(&sth->item[i].addr, PROXY_TYPE_SOCKS_V4, 0);
                  DISABLE_PFD(pfd);
                  continue;
               } else
                  log_error("socks4in: (rejected) %s %02X %02X %02X %02X %02X %02X %02X %02X",
                      inet_ntoa(sth->item[i].addr), socks4in.vn, socks4in.cd,
                      socks4in.dstport[0], socks4in.dstport[1],
                      socks4out.dstip[0], socks4out.dstip[1],
                      socks4out.dstip[2], socks4out.dstip[3]);
            }
            Debug((DBLMAX, "scan_socks4: %s recv_nonb done", inet_ntoa(sth->item[i].addr)));
            sth->item[i].state |= SIT_NOTFOUND;
            DISABLE_PFD(pfd);
         }
      }
   }
      
   //Debug((DBLMAX, "scan_socks: (SOCKS 4) closing fd's"));
   scanItems_closeall(sth);
   
   /************************* SOCKS 5 CHECK ********************************/
      
   scanItems_init(sth, POLLWRITEFLAGS, SOCKS_PORT);
   
   //Debug((DBLMAX, "scan_socks (SOCKS 5) sscanItems_init ok"));
   
   for( ; ; ) {
      
      nfds = poll(sth->poll_fds, sth->counter, POLL_TIMEOUT);
      
      if(nfds <= -1) {
         if(errno == EINTR)
            continue;
         log_error("scan_socks5: poll error %d\n", errno);
         break;
      }
      
      /* Timeout checking */
      time(&now);
      timed_out = true;
      for(i = 0; i < sth->counter; ++i) {
         if(sth->item[i].state & (SIT_INACTIVE|SIT_FOUND))
            continue;
         
         if(!(sth->item[i].state & SIT_TIMEOUT) && (now >= sth->item[i].timeout))
         {
            sth->item[i].state |= SIT_TIMEOUT;
         }
         
         timed_out = timed_out && ( sth->item[i].state & SIT_TIMEOUT );
      }
      
      if(timed_out) {
         /* Every fds are timed out, exiting. */
         //Debug((DBLMAX, "scan_socks5: all timed out"));
         break;
      }
      
      if(nfds == 0)
         continue;
      
      pfd = sth->poll_fds;
      
      for (i = 0; i < sth->counter; ++i, ++pfd) {
         if(pfd->revents & POLLERRORS) {
            sth->item[i].state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            continue;
         }
         
         if(sth->item[i].state & (SIT_INACTIVE|SIT_FOUND))
            continue;
         
         if((err = get_sockerr(pfd->fd))) {
            sth->item[i].state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            continue;
         }
         
         if((pfd->events & POLLWRITEFLAGS) && (pfd->revents & POLLWRITEFLAGS)) {
            time(&sth->item[i].timeout);
            sth->item[i].timeout += MyPerform->timeout_send;
            
            if(send_nonb(pfd->fd, (char *) &socks5out, sizeof(struct socks5), &length) < 0) {
               sth->item[i].state |= SIT_ERROR;
               DISABLE_PFD(pfd);
               continue;
            }
            
            pfd->events = POLLREADFLAGS;   /* Now we want to read the reply. */
            pfd->revents = 0;
         }
         
         if((pfd->events & POLLREADFLAGS) && (pfd->revents & POLLREADFLAGS))
         {
            memset((void *)&socks5in, 0, sizeof(struct socks5_response));
            if(recv_nonb(pfd->fd, (char *) &socks5in, sizeof(struct socks5_response), &length) < 0)
            {
               sth->item[i].state |= SIT_ERROR;
               DISABLE_PFD(pfd);
               continue;
            }
            
            if (length > 1) {
               if (socks5in.vn == S5_VN) {
                  if (socks5in.method == S5_R_METHOD_NOAUTH) {
                     
                     sth->item[i].state |= SIT_FOUND;
                     /* Toast it. */
                     proxy_toast(&sth->item[i].addr, PROXY_TYPE_SOCKS_V5, 0);
                     
                     Debug((DBLBASIC, "%d %s > PROXY_TYPE_SOCKS_V5 NOAUTH",
                         pthread_self(), inet_ntoa(sth->item[i].addr)));
                     DISABLE_PFD(pfd);
                     continue;
                     
                   } else if(socks5in.method == S5_R_METHOD_GSSAPI) {
                      sendto_console("[%s] SOCKS 5 GSSAPI (ok) found on %s\r\n",
                           Me->id, inet_ntoa(sth->item[i].addr));
                   } else if(socks5in.method == S5_R_METHOD_USERPW) {
                      sendto_console("[%s] SOCKS 5 USER/PW (ok) found on %s\r\n",
                           Me->id, inet_ntoa(sth->item[i].addr));
                   } else {
                      log_error("socks5in: (rejected) %s %02X %02X",
                           inet_ntoa(sth->item[i].addr), socks5in.vn, socks5in.method);
                      Debug((DBLMAX, "scan_socks:socks5: no valid type"));
                   }
                }
            }
            sth->item[i].state |= SIT_NOTFOUND;
            DISABLE_PFD(pfd);
         }
      }
   }
   
   /* Remove SIT_NOSOCKS flag (used to not try to connect twice in socks check) */
   for(i = 0; i < sth->counter; i++) {
      if(sth->item[i].state & SIT_NOSOCKS)
            sth->item[i].state &= ~SIT_NOSOCKS;
   }
   
   scanItems_closeall(sth);
}


void
get_socks4_dest_ip(struct socks4 * outdata)
{
   int i, chooser = (int) ((double)totalTarget*rand()/(RAND_MAX+1.0));
   Target_s *target = MyTarget;
   
   for (i = chooser; target && i--; target = target->next)
      ;
   
   if (!target) /* should never happen */
      target = MyTarget;
   
   memcpy(&outdata->dstip, &target->addr, sizeof(outdata->dstip));
   Debug((DBLMAX, "mp_socks:get_socks4_dest_ip: %u.%u.%u.%u",
            outdata->dstip[0], outdata->dstip[1], outdata->dstip[2], outdata->dstip[3])); 
}

/* mp_socks.c */
