/* Pxys, An IRC ProxyScan, mp_utils.h
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * $Id: mp_utils.c,v 1.4 2001/11/04 01:53:37 mbuna Exp $
 */
 
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>   /* strerror */
#include <unistd.h>   /* close */
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#if HAVE_FCNTL_H == 1
#   include <fcntl.h>
#endif
#if HAVE_PTHREAD_H == 1
#   include <pthread.h>
#endif
#if HAVE_SYS_POLL_H == 1
#include <sys/poll.h>
#endif

#include <sys/ioctl.h>
#include <errno.h>

#include "typedefs.h"
#include "conf.h"
#include "debug.h"

#include "mp_utils.h"

#include "m_structs.h"
#include "m_log.h"
#include "m_shutdown.h"

#include "ms_msgout.h"

void
scanItems_init(sth, events, port)
scanthread_s * sth;
int events;
unsigned short port;
{
   int fd, i, optval;
   struct sockaddr_in localAddr;
   
   if(Me->vhost) {
      memset(&localAddr, 0, sizeof(struct sockaddr_in));
      localAddr.sin_family = AF_INET;
      localAddr.sin_addr.s_addr = Me->bindaddr;
      localAddr.sin_port = htons(0);
   }
   
   for (i=0; i < sth->counter; i++) {
      if(sth->item[i].state & (SIT_INACTIVE|SIT_FOUND|SIT_NOSOCKS)) {
         sth->poll_fds[i].fd = -1;
         continue;
      }
      
      /* Create socket */
      fd = socket(AF_INET, SOCK_STREAM, 0);
      if (fd < 0) {
         optval = errno;
         sendto_console("[%s] socket() failed (%s)", Me->id, strerror(optval));
         scanItems_closeall(sth);
         log_error("mp_utils:scanItems_init: cannot open socket, aborting. (errno = %d - %s)",
              optval, strerror(optval));
         m_shutdown();
         return;
      }
      
      sth->poll_fds[i].fd = fd;
      sth->poll_fds[i].events = events;
      sth->poll_fds[i].revents = 0;
      
      /* Are we using a vhost? if so, bind localAddr any port number */
      if (Me->vhost) {
         if (bind(fd, (struct sockaddr *) &localAddr, sizeof(localAddr)) < 0) {
            log_error("mp_utils:scanItems_init: bind failed.");
            m_shutdown();
            return;
         }
      }
      
      memset(&sth->item[ i ].clientAddr, 0, sizeof(sth->item[i].clientAddr));
      sth->item[i].clientAddr.sin_family = AF_INET;
      sth->item[i].clientAddr.sin_addr.s_addr = sth->item[i].addr.s_addr;
      sth->item[i].clientAddr.sin_port = htons(port);
      time(&sth->item[i].timeout); /* Reentrant */
      sth->item[i].timeout += MyPerform->timeout_connect;
      sth->item[i].tryread = 0;
      
      if (set_nonblock_flag(fd) == -1) {
         sth->item[ i ].state |= SIT_ERROR;
         log_error("mp_utils:scanItems_init:set_nonblock_flag failed (errno == %d)", errno);
         Debug((DBLMAX, "mp_utils:scanItems_init:set_nonblock_flag failed (errno == %d)", errno));
         /* TODO? */
         continue;
      }

#if defined( SOL_SOCKET ) && defined( SO_REUSEADDR )
      optval = 1;
      (void)setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
#endif
      
      if(!connect_nonb(fd, &sth->item[i].clientAddr)) {
         sth->item[i].state |= SIT_ERROR;
         /*
         Debug((DBLMAX, "mp_utils:scanItems_init:connect_nonb failed (errno == %d - %s)",
             errno, strerror(errno)));
             */
         continue;
      }
   }

}

void
scanItems_closeall(scanthread_s * sth)
{
   int i;
   unsigned short state;
   
   for(i=0; i < sth->counter; i++)
   {
      state = sth->item[i].state;
      
      if(sth->poll_fds[i].fd != -1) {
         /* shutdown */
         (void) shutdown(sth->poll_fds[i].fd, 2);
      
         /* close fd */
         myclose(sth->poll_fds[i].fd);
      }
      
      /* reset flags */
      if(state & SIT_ACTIVE) {
         /* reset some volatile flags :) */
         if(sth->item[i].state & SIT_TIMEOUT)
            sth->item[i].state &= ~SIT_TIMEOUT;
         if(sth->item[i].state & SIT_ERROR)
            sth->item[i].state &= ~SIT_ERROR;
         if(sth->item[i].state & SIT_INPROGRESS)
            sth->item[i].state &= ~SIT_INPROGRESS;
      }
   }
}


int
set_nonblock_flag(int desc)
{
   int res = 1;
   int oldflags = fcntl(desc, F_GETFL, 0);
   if(oldflags < 0)
      return -1;
#ifdef O_NDELAY
   oldflags |= O_NDELAY;
#elif defined( O_NONBLOCK )
   oldflags |= O_NONBLOCK;
#else
#  error unknown fcntl O_NONBLOCK flag
#endif

#ifdef FIONBIO
   if (ioctl(desc, FIONBIO, &res) == -1)
      log_error("set_nonblock_flag> ioctl failed (%d)\n", errno);
#else
#  warning unknown ioctl FIONBIO option
#endif
   
   return fcntl(desc, F_SETFL, oldflags);
}

int
get_sockerr(int fd)
{
  int    err = 0;
#if defined(SO_ERROR)
  unsigned int len = sizeof(err);
  getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &len);
#endif
  return err;
}

int
connect_nonb(int fd, const struct sockaddr_in* sin)
{
  if (connect(fd, (struct sockaddr*) sin, sizeof(struct sockaddr_in))) {
     if (errno != EINPROGRESS)
        return 0;
  }
  return 1;
}

int
set_sockbufs(int fd, unsigned int size)
{
  unsigned int opt = size;
  return (0 == setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (const char*) &opt, sizeof(opt)) &&
          0 == setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (const char*) &opt, sizeof(opt)));
}

int
recv_nonb(int fd, char* buf, unsigned int length, unsigned int* count_out)
{
  int res;
  *count_out = 0;
  errno = 0;

  if (0 < (res = recv(fd, buf, length, 0))) {
    *count_out = (unsigned int) res;
    return 1;
  } else if (res < 0) {
    if (EWOULDBLOCK == errno || EAGAIN == errno)
      return 0;
  } 
  return -1;
}

int
send_nonb(int fd, const char* buf, unsigned int length, unsigned int* count_out)
{
  int res;
  *count_out = 0;
  errno = 0;

  if (-1 < (res = send(fd, buf, length, 0))) {
    *count_out = (unsigned int) res;
    return 1;
  }
  else if(EWOULDBLOCK == errno || EAGAIN == errno || ENOMEM == errno || ENOBUFS == errno)
     return 0;
  return -1;
}


void
myclose(int fd)
{
   int tryed = 0;
   
   while (close(fd) == -1) {
      
      Debug((DBLMAX, "close(fd) failed (%d)...", errno));
      
      if(EBADF == errno)
         break;
      
      /* usleep(50000); --not nice */
      
      if(++tryed >= 10) {
         log_error("mp_utils:myclose error (errno == %d)", errno);
         Debug((DBLMAX, "close(fd) failed tryed (%d) again...", tryed));
         break;
      }
   }
}

/* mp_utils.c */
