/* Pxys - An IRC ProxyScan, xmalloc.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: xmalloc.c,v 1.2 2001/10/17 21:53:11 mbuna Exp $
 */

#include "config.h"
#include "xmalloc.h"
#include "debug.h"
#include "m_log.h"
#include "m_shutdown.h"

#include <stdlib.h>


void*
xmalloc(size_t size)
{
   void *ptr = malloc(size);
   
   if (ptr == NULL) {
      Debug((DBLMAX, "xmalloc: out of memory (allocating %d bytes)", (int)size));
      log_error("xmalloc: out of memory (allocating %d bytes)\n", (int)size);
      m_shutdown();
   }
   
   return ptr;
}

void*
xcalloc(size_t number, size_t size)
{
   void *ptr = calloc(number, size);
   
   if (ptr == NULL) {
      Debug((DBLMAX, "xcalloc: out of memory (allocating %d objects of %d bytes)",
         (int)number, (int)size));
      log_error("xcalloc: out of memory (allocating %d objects of %d bytes)\n",
         (int)number, (int)size);
         
      m_shutdown();
   }
   
   return ptr;
}

void
xfree(void *ptr)
{
   if (ptr == NULL) {
      Debug((DBLMAX, "xfree: NULL pointer given as argument"));
      log_error("xfree: NULL pointer given as argument\n");
      m_shutdown();
   }
   
   free(ptr);
}

